import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The Desert serves as the World for this scenario.
 * 
 * @author Jordan Cohen 
 * @version 2013
 */
public class Desert extends World
{
    // instance variables
    private int counter = 0;
    private int spawnRate;
    private int spawnY;
    private int vehicleSpeed;

    private int lives = 3;

    private int level;
    private int autoCount;
    private Level currentLevel;
    private boolean levelLoaded = false;
    private boolean spawning;

    private boolean actionPaused = false;
    private boolean showScore = false;
    private boolean deathSequenceActive = false;

    // Graphics loading
    private GreenfootImage frogImage = new GreenfootImage ("frog.png");    
    private GreenfootImage splat = new GreenfootImage ("splat.png");

    // Sounds
    private GreenfootSound smush;
    private GreenfootSound levelUpSound;
    private GreenfootSound gameOverSound;

    // owned instance objects
    private LifeCounter lifeCounter = new LifeCounter ();
    private LevelCounter levelCounter = new LevelCounter ();
    private Frogger myFrog = new Frogger();

    /**
     * Constructor for objects of class Desert.
     */
    public Desert()
    {    
        // Create a new world with 600x400 cells with a cell size of 1x1 pixels.
        super(600, 400, 1); 

        // Ensure that Frogs and Blood occur over top of everything else
        setPaintOrder(Frogger.class, BloodPool.class, ScoreBoard.class, Enemy.class);

        // Add the life counter to the World
        addObject (lifeCounter, 67, 25);
        addObject (levelCounter, 530, 24);

        // Initialize sound within the constructor to avoid wierd sound lag
        smush = new GreenfootSound("smush.wav");
        levelUpSound = new GreenfootSound("mariocoin.mp3");
        gameOverSound = new GreenfootSound("GameOver.wav");

        // Set level to 1
        level = 1;
        autoCount = 0;
    }

    public void act ()
    {
        checkIfLevelLoadRequired();
        checkSpawnAndDeath();
        checkKeys();
    }

    // Method to set necessary values to change level
    private void levelLoader ()
    {
        // Add a fresh Frogger object
        addObject(myFrog, 300, 370);

        // Update the info stored in currentLevel
        currentLevel = new Level (level);

        levelCounter.setLevel(level);

        // Set image to frog (in case "splat" was being displayed)
        myFrog.setImage(frogImage);

        // change appropriate booleans for game flow
        actionPaused = false;
        spawning = true;

        // load instance variables from Level
        spawnRate = currentLevel.getSpawnRate();
        vehicleSpeed = currentLevel.getVehicleSpeed();

        myFrog.setSpeed(currentLevel.getFroggerSpeed());

        // Ensure that level is not loaded again until necessary
        levelLoaded = true;
    }

    /**
     * Controlled spawning of new cars
     */
    private void spawnCars()
    {
        int randVal = Greenfoot.getRandomNumber(3)+1;
        int spawnChance = Greenfoot.getRandomNumber(100) + 1;
        // spawn single
        if (counter % spawnRate == 0)
        {
            if (randVal == 1)
                spawnY = 127;
            else if (randVal == 2)
                spawnY = 223;
            else
                spawnY = 320;
            addObject(new EastboundAuto(vehicleSpeed), 20, spawnY);
            autoCount++;
        }
        else if (counter % spawnRate == spawnRate / 2)
        {
            if (randVal == 1)
                spawnY = 78;
            else if (randVal == 2)
                spawnY = 173;
            else
                spawnY = 270;
            addObject(new WestboundAuto(vehicleSpeed), 578, spawnY);
            autoCount++;
        }
        counter++;
    }

    /**
     * Check if a new level needs to be loaded
     */
    private void checkIfLevelLoadRequired()
    {
        if (levelLoaded == false)
        {
            // Load level if player has lives left
            if (lives > 0)
            {
                levelLoader();
                levelLoaded = true;
            }
            else
            {
                ScoreBoard s = new ScoreBoard (level, "Game Over", "On Level ");
                addObject (s, 300,200);
                gameOverSound.play();
                // End program
                Greenfoot.stop();
            }
        } 
    }

    /**
     * Evaluate variables related to spawning and death and act 
     * appropriately
     */
    private void checkSpawnAndDeath()
    {
        // If appropriate, spawn cars
        if (actionPaused == false && spawning == true)
        {
            spawnCars();
        }
        // Control sequence to ensure that after death, program waits until
        // all cars are off-screen before spawning the next Frogger
        if (deathSequenceActive)
        {
            if (autoCount == 0)
            {
                deathSequenceActive = false;
                levelLoaded = false;
            }
        }
    }

    private void checkKeys()
    {
        
        if (myFrog.isActive() == true)
        {
            int speed = myFrog.getSpeed();
            
            if (Greenfoot.isKeyDown("up"))
            {
                myFrog.setLocation (myFrog.getX(), myFrog.getY() - speed);
            }
            if (Greenfoot.isKeyDown("down"))
            {
                myFrog.setLocation (myFrog.getX(), myFrog.getY() + speed);
            }
            else if (Greenfoot.isKeyDown("left"))
            {
                myFrog.setLocation (myFrog.getX() - speed, myFrog.getY());
            }
            else if (Greenfoot.isKeyDown("right"))
            {
                myFrog.setLocation (myFrog.getX()  + speed, myFrog.getY());
            }
        }
    }

    /**
     * This method gets called when the player dies or finishes a level
     * 
     * True = player completed level
     * False = player died
     * 
     * @param   win     boolean that represents whether player has won or died
     */
    public void endLevel(boolean win)
    {
        // Set variables states to indicate that game is ready to change to 
        // next level
        ScoreBoard s = null;
        actionPaused = true;
        spawning = false;

        // If the player beat the level...
        if (win == true)
        {
            // increase current level count
            level++;
            // play sound to indicate level completed
            levelUpSound.play();
            // remove this frog
            removeObject (myFrog);
            // set boolean to trigger loading next level
            levelLoaded = false;
            // a short delay while the sound plays for effect
            Greenfoot.delay(50);

        }
        // Otherwise, if the player died...
        else
        {
            // Change the Frogger's image to the bloody frog
            myFrog.setImage(splat);
            // Play a smush sound
            smush.play();

            // Make a bloody effect
            BloodPool b = new BloodPool();
            addObject (b, myFrog.getX(), myFrog.getY());

            // Subtract from lives
            lives -= 1;
            // Call the Actore lifeCounter to visually display new number
            // of lives (see method for more details)
            lifeCounter.subtractLife();
            actionPaused = false;

            // Set bolean to trigger the death sequence which resets player
            // position
            deathSequenceActive = true;

            // Remove this frog, as it is dead
            removeObject (myFrog);
        }
    }

    public boolean isActionPaused ()
    {
        return actionPaused;
    }

    public void reduceAutoCount (int byHowMany)
    {
        autoCount -= byHowMany;
    }

}
